@@ -5,7 +5,8 @@ This is the main class for file explorer (datasets).
 from dataclasses import dataclass
 from typing import Union
 from cloudos_cli.clos import Cloudos
-from cloudos_cli.utils.requests import retry_requests_get, retry_requests_put
+from cloudos_cli.utils.errors import BadRequestException
+from cloudos_cli.utils.requests import retry_requests_get, retry_requests_put, retry_requests_post, retry_requests_delete
 import json
 
 @dataclass
@@ -42,10 +43,7 @@ class Datasets(Cloudos):
     def project_id(self, v) -> None:
         if isinstance(v, property):
             # Fetch the value as not defined by user.
-            self._project_id = self.fetch_cloudos_id(
-                self.apikey,
-                self.cloudos_url,
-                'projects',
+            self._project_id = self.fetch_project_id(
                 self.workspace_id,
                 self.project_name,
                 verify=self.verify)
@@ -53,39 +51,18 @@ class Datasets(Cloudos):
             # Let the user define the value.
             self._project_id = v
 
-    def fetch_cloudos_id(self,
-                         apikey,
-                         cloudos_url,
-                         resource,
+    def fetch_project_id(self,
                          workspace_id,
-                         name,
-                         mainfile=None,
-                         importsfile=None,
-                         repository_platform='github',
+                         project_name,
                          verify=True):
-        """Fetch the cloudos id for a given name.
+        """Fetch the project id for a given name.
 
         Parameters
         ----------
-        apikey : string
-            Your CloudOS API key
-        cloudos_url : string
-            The CloudOS service url.
-        resource : string
-            The resource you want to fetch from. E.g.: projects.
         workspace_id : string
             The specific Cloudos workspace id.
-        name : string
-            The name of a CloudOS resource element.
-        mainfile : string
-            The name of the mainFile used by the workflow. Only used when resource == 'workflows'.
-            Required for WDL pipelines as different mainFiles could be loaded for a single
-            pipeline.
-        importsfile : string
-            The name of the importsFile used by the workflow. Optional and only used for WDL pipelines
-            as different importsFiles could be loaded for a single pipeline.
-        repository_platform : string
-            The name of the repository platform of the workflow resides.
+        project_name : string
+            The name of a CloudOS project element.
         verify: [bool|string]
             Whether to use SSL verification or not. Alternatively, if
             a string is passed, it will be interpreted as the path to
@@ -96,37 +73,8 @@ class Datasets(Cloudos):
         project_id : string
             The CloudOS project id for a given project name.
         """
-        allowed_resources = ['projects', 'workflows']
-        if resource not in allowed_resources:
-            raise ValueError('Your specified resource is not supported. ' +
-                             f'Use one of the following: {allowed_resources}')
-        if resource == 'workflows':
-            content = self.get_workflow_list(workspace_id, verify=verify)
-            for element in content:
-                if (element["name"] == name and element["workflowType"] == "docker" and
-                        not element["archived"]["status"]):
-                    return element["_id"]  # no mainfile or importsfile
-                if (element["name"] == name and
-                        element["repository"]["platform"] == repository_platform and
-                        not element["archived"]["status"]):
-                    if mainfile is None:
-                        return element["_id"]
-                    elif element["mainFile"] == mainfile:
-                        if importsfile is None and "importsFile" not in element.keys():
-                            return element["_id"]
-                        elif "importsFile" in element.keys() and element["importsFile"] == importsfile:
-                            return element["_id"]
-        elif resource == 'projects':
-            content = self.get_project_list(workspace_id, verify=verify)
-            # New API projects endpoint spec
-            for element in content:
-                if element["name"] == name:
-                    return element["_id"]
-        if mainfile is not None:
-            raise ValueError(f'[ERROR] A workflow named \'{name}\' with a mainFile \'{mainfile}\'' +
-                             f' and an importsFile \'{importsfile}\' was not found')
-        else:
-            raise ValueError(f'[ERROR] No {name} element in {resource} was found')
+        return self.get_project_id_from_name(workspace_id, project_name, verify=verify)
+
     def list_project_content(self):
         """
         Fetch the information of the directories present in the projects.
@@ -150,6 +98,8 @@ class Datasets(Cloudos):
                                                                                   self.project_id,
                                                                                   self.workspace_id),
                                headers=headers, verify=self.verify)
+        if r.status_code >= 400:
+            raise BadRequestException(r)
         raw = r.json()
         datasets = raw.get("datasets", [])
         #  Normalize response
@@ -195,6 +145,8 @@ class Datasets(Cloudos):
                                                                               folder_id,
                                                                               self.workspace_id),
                                 headers=headers, verify=self.verify)
+        if r.status_code >= 400:
+            raise BadRequestException(r)
         return r.json()
 
     def list_s3_folder_content(self, s3_bucket_name, s3_relative_path):
@@ -224,6 +176,8 @@ class Datasets(Cloudos):
                                                                                                              s3_relative_path,
                                                                                                              self.workspace_id),
                                 headers=headers, verify=self.verify)
+        if r.status_code >= 400:
+            raise BadRequestException(r)
         raw = r.json()
 
         #  Normalize response
@@ -237,7 +191,7 @@ class Datasets(Cloudos):
             else:
                 item["s3Prefix"] = item['path']
                 item["s3BucketName"] = s3_bucket_name
-
+                item["fileType"] = "S3File"
                 normalized["files"].append(item)
 
         return normalized
@@ -265,8 +219,63 @@ class Datasets(Cloudos):
                                                                                      folder_id,
                                                                                      self.workspace_id),
                                 headers=headers, verify=self.verify)
+        if r.status_code >= 400:
+            raise BadRequestException(r)
         return r.json()
     
+    def list_azure_container_content(self, container_name: str, storage_account_name: str, path: str):
+        """
+        List contents of an Azure Blob container path.
+        """
+        headers = {
+            "Content-type": "application/json",
+            "apikey": self.apikey
+        }
+
+        url = f"{self.cloudos_url}/api/v1/data-access/azure/container-contents"
+        url += f"?containerName={container_name}&storageAccountName={storage_account_name}"
+        url += f"&path={path}&teamId={self.workspace_id}"
+
+        r = retry_requests_get(url, headers=headers, verify=self.verify)
+        if r.status_code >= 400:
+            raise BadRequestException(r)
+        raw = r.json()
+
+        # Normalize response to match existing expectations
+        normalized = {"folders": [], "files": []}
+        for item in raw.get("contents", []):
+            is_dir = item.get("isDir", False)
+
+            # Set a name field based on the last part of the blob path
+            path_str = item.get("path", "")
+            name = item.get("name") or path_str.rstrip("/").split("/")[-1]
+
+            # inject expected structure
+            if is_dir:
+                normalized["folders"].append({
+                    "_id": item.get("_id"),
+                    "name": name,
+                    "folderType": "AzureBlobFolder",
+                    "blobPrefix": path_str,
+                    "blobContainerName": container_name,
+                    "blobStorageAccountName": storage_account_name,
+                    "kind": "Folder" 
+                })
+            else:
+                normalized["files"].append({
+                    "_id": item.get("_id"),
+                    "name": name,
+                    "fileType": "AzureBlobFile",
+                    "blobName": path_str,
+                    "blobContainerName": container_name,
+                    "blobStorageAccountName": storage_account_name,
+                    "sizeInBytes": item.get("size", 0),
+                    "updatedAt": item.get("lastModified"),
+                    "kind": "File" 
+                })
+
+        return normalized
+
     def list_folder_content(self, path=None):
         """
         Wrapper to list contents of a CloudOS folder.
@@ -324,6 +333,21 @@ class Datasets(Cloudos):
                             path_depth += 1
                             break
 
+                    elif folder_type == "AzureBlobFolder":
+                        container_name = job_folder['blobContainerName']
+                        storage_account_name = job_folder['blobStorageAccountName']
+                        blob_prefix = job_folder['blobPrefix']
+                        # trailing slash is mandatory for azure, otherwise it will not list the content of thefolde, just the folder 
+                        if not blob_prefix.endswith('/'):
+                            blob_prefix += '/'
+
+                        if path_depth == len(parts) - 1:
+                            return self.list_azure_container_content(container_name, storage_account_name, blob_prefix)
+                        else:
+                            sub_path = '/'.join(parts[0:path_depth+1])
+                            folder_content = self.list_folder_content(sub_path)
+                            path_depth += 1
+                            break
                     else:
                         raise ValueError(f"Unsupported folder type '{folder_type}' for path '{path}'")
 
@@ -366,4 +390,187 @@ class Datasets(Cloudos):
             }
         }
         response = retry_requests_put(url, headers=headers, data=json.dumps(payload), verify=self.verify)
+        if response.status_code >= 400:
+            raise BadRequestException(response)
+        return response
+
+    def rename_item(self, item_id: str, new_name: str, kind: str):
+        """
+        Rename a file or folder in CloudOS.
+
+        Parameters
+        ----------
+        item_id : str
+            The ID of the file or folder to rename.
+        new_name : str
+            The new name to assign to the item.
+        kind : str
+            Either "File" or "Folder"
+
+        Returns
+        -------
+        response : requests.Response
+            The response object from the CloudOS API.
+        """
+        if kind not in ("File", "Folder"):
+            raise ValueError("Invalid kind provided. Must be 'File' or 'Folder'.")
+
+        endpoint = "files" if kind == "File" else "folders"
+        url = f"{self.cloudos_url}/api/v1/{endpoint}/{item_id}?teamId={self.workspace_id}"
+
+        headers = {
+            "accept": "application/json",
+            "content-type": "application/json",
+            "ApiKey": self.apikey
+        }
+
+        payload = {
+            "name": new_name
+        }
+
+        response = retry_requests_put(url, headers=headers, data=json.dumps(payload), verify=self.verify)
+        if response.status_code >= 400:
+            raise BadRequestException(response)
+        return response
+    
+    def copy_item(self, item, destination_id, destination_kind):
+        """Copy a file or folder (S3, Azure or Virtual) to a destination in CloudOS."""
+        headers = {
+            "accept": "application/json",
+            "content-type": "application/json",
+            "ApiKey": self.apikey
+        }
+        parent = {"kind": destination_kind, "id": destination_id}
+
+        # Virtual folder
+        if item.get("folderType") == "VirtualFolder":
+            payload = {
+                "copyContentsFrom": item["_id"],
+                "name": item["name"],
+                "parent": parent
+            }
+            url = f"{self.cloudos_url}/api/v1/folders/virtual?teamId={self.workspace_id}"
+        # S3 folder
+        elif item.get("folderType") == "S3Folder":
+            payload = {
+                "s3BucketName": item["s3BucketName"],
+                "s3ObjectKey": item.get("s3ObjectKey") or item.get("s3Prefix"),
+                "name": item["name"],
+                "parent": parent,
+                "isManagedByLifebit": item.get("isManagedByLifebit", False)
+            }
+            url = f"{self.cloudos_url}/api/v1/folders/s3?teamId={self.workspace_id}"
+        # S3 file
+        elif item.get("fileType") == "S3File":
+            payload = {
+                "s3BucketName": item["s3BucketName"],
+                "s3ObjectKey": item.get("s3ObjectKey") or item.get("s3Prefix"),
+                "name": item["name"],
+                "parent": parent,
+                "isManagedByLifebit": item.get("isManagedByLifebit", False),
+                "sizeInBytes": item.get("sizeInBytes", 0)
+            }
+            url = f"{self.cloudos_url}/api/v1/files/s3?teamId={self.workspace_id}"
+        # Azure folder
+        elif item.get("folderType") == "AzureBlobFolder":
+            payload = {
+                "blobContainerName": item["blobContainerName"],
+                "blobPrefix": item["blobPrefix"],
+                "blobStorageAccountName": item["blobStorageAccountName"],
+                "name": item["name"],
+                "parent": parent
+            }
+            url = f"{self.cloudos_url}/api/v1/folders/azure-blob?teamId={self.workspace_id}"
+        # Azure file
+        elif item.get("fileType") == "AzureBlobFile":
+            payload = {
+                "blobContainerName": item["blobContainerName"],
+                "blobName": item["blobName"],
+                "blobStorageAccountName": item["blobStorageAccountName"],
+                "name": item["name"],
+                "parent": parent,
+                "isManagedByLifebit": item.get("isManagedByLifebit", False),
+                "sizeInBytes": item.get("sizeInBytes", 0)
+            }
+            url = f"{self.cloudos_url}/api/v1/files/azure-blob?teamId={self.workspace_id}"
+
+        else:
+            raise ValueError(f"Unknown item type for copy: {item.get('name')}")
+        response = retry_requests_post(url, headers=headers, json=payload)
+        if response.status_code >= 400:
+            raise BadRequestException(response)
+        return response
+    
+    def create_virtual_folder(self, name: str, parent_id: str, parent_kind: str):
+        """
+        Create a new virtual folder in CloudOS under a given parent.
+
+        Parameters
+        ----------
+        name : str
+            The name of the new folder.
+        parent_id : str
+            The ID of the parent (can be a Dataset or a Folder).
+        parent_kind : str
+            The type of the parent: either "Dataset" or "Folder".
+
+        Returns
+        -------
+        response : requests.Response
+            The response object from the CloudOS API.
+        """
+        if parent_kind not in ("Dataset", "Folder"):
+            raise ValueError("Invalid parent_kind. Must be 'Dataset' or 'Folder'.")
+
+        url = f"{self.cloudos_url}/api/v1/folders/virtual?teamId={self.workspace_id}"
+
+        headers = {
+            "accept": "application/json",
+            "content-type": "application/json",
+            "ApiKey": self.apikey
+        }
+
+        payload = {
+            "name": name,
+            "parent": {
+                "kind": parent_kind,
+                "id": parent_id
+            }
+        }
+
+        response = retry_requests_post(url, headers=headers, json=payload, verify=self.verify)
+        if response.status_code >= 400:
+            raise BadRequestException(response)
+        return response
+    
+    def delete_item(self, item_id: str, kind: str):
+        """
+        Delete a file or folder in CloudOS.
+
+        Parameters
+        ----------
+        item_id : str
+            The ID of the file or folder to delete.
+        kind : str
+            Must be either "File" or "Folder".
+
+        Returns
+        -------
+        response : requests.Response
+            The response object from the CloudOS API.
+        """
+        if kind not in ("File", "Folder"):
+            raise ValueError("Invalid kind provided. Must be 'File' or 'Folder'.")
+
+        endpoint = "files" if kind == "File" else "folders"
+        url = f"{self.cloudos_url}/api/v1/{endpoint}/{item_id}?teamId={self.workspace_id}"
+
+        headers = {
+            "accept": "application/json",
+            "ApiKey": self.apikey
+        }
+
+        response = retry_requests_delete(url, headers=headers, verify=self.verify)
+        if response.status_code >= 400:
+            raise BadRequestException(response)
         return response
\ No newline at end of file
